import socket
import logging
from time import time, sleep

from ogn.client import settings


def create_aprs_login(user_name, pass_code, app_name, app_version, aprs_filter=None):
    if not aprs_filter:
        return "user {} pass {} vers {} {}\n".format(user_name, pass_code, app_name, app_version)
    else:
        return "user {} pass {} vers {} {} filter {}\n".format(user_name, pass_code, app_name, app_version, aprs_filter)


class AprsClient:
    def __init__(self, aprs_user, aprs_filter='', settings=settings):
        self.logger = logging.getLogger(__name__)
        self.logger.info("Connect to OGN as {} with filter '{}'".format(aprs_user, (aprs_filter if aprs_filter else 'full-feed')))
        self.aprs_user = aprs_user
        self.aprs_filter = aprs_filter
        self.settings = settings

        self._kill = False

    def connect(self):
        # create socket, connect to server, login and make a file object associated with the socket
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

        if self.aprs_filter:
            port = self.settings.APRS_SERVER_PORT_CLIENT_DEFINED_FILTERS
        else:
            port = self.settings.APRS_SERVER_PORT_FULL_FEED

        self.sock.connect((self.settings.APRS_SERVER_HOST, port))
        self.logger.debug('Server port {}'.format(port))

        login = create_aprs_login(self.aprs_user, -1, self.settings.APRS_APP_NAME, self.settings.APRS_APP_VER, self.aprs_filter)
        self.sock.send(login.encode())
        self.sock_file = self.sock.makefile('rw')

        self._kill = False

    def disconnect(self):
        self.logger.info('Disconnect')
        try:
            # close everything
            self.sock.shutdown(0)
            self.sock.close()
        except OSError:
            self.logger.error('Socket close error', exc_info=True)

        self._kill = True

    def run(self, callback, timed_callback=lambda client: None, autoreconnect=False):
        while not self._kill:
            try:
                keepalive_time = time()
                while not self._kill:
                    if time() - keepalive_time > self.settings.APRS_KEEPALIVE_TIME:
                        self.logger.info('Send keepalive')
                        self.sock.send('#keepalive\n'.encode())
                        timed_callback(self)
                        keepalive_time = time()

                    # Read packet string from socket
                    packet_str = self.sock_file.readline().strip()

                    # A zero length line should not be return if keepalives are being sent
                    # A zero length line will only be returned after ~30m if keepalives are not sent
                    if len(packet_str) == 0:
                        self.logger.warning('Read returns zero length string. Failure.  Orderly closeout')
                        break

                    callback(packet_str)
            except ConnectionError:
                self.logger.error('ConnectionError', exc_info=True)
            except socket.error:
                self.logger.error('socket.error', exc_info=True)
            except UnicodeDecodeError:
                self.logger.error('UnicodeDecodeError', exc_info=True)

            if autoreconnect and not self._kill:
                self.connect()
            else:
                return


class TelnetClient:
    def __init__(self, settings=settings):
        self.logger = logging.getLogger(__name__)
        self.logger.info("Connect to local telnet server")
        self.settings = settings

    def connect(self):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((self.settings.TELNET_SERVER_HOST, self.settings.TELNET_SERVER_PORT))

    def run(self, callback, autoreconnect=False):
        while True:
            try:
                self.sock_file = self.sock.makefile(mode='rw', encoding='iso-8859-1')
                while True:
                    packet_str = self.sock_file.readline().strip()
                    callback(packet_str)

            except ConnectionRefusedError:
                self.logger.error('Telnet server not running', exc_info=True)

            if autoreconnect:
                sleep(1)
                self.connect()
            else:
                return

    def disconnect(self):
        self.logger.info('Disconnect')
        self.sock.shutdown(0)
        self.sock.close()



import unittest

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.aprs_comment.skylines_parser import SkylinesParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = SkylinesParser().parse_position("id2816 -015fpm")

        self.assertEqual(message['address'], "2816")
        self.assertAlmostEqual(message['climb_rate'], -15 * FPM_TO_MS, 2)


if __name__ == '__main__':
    unittest.main()



import re

from ogn.parser.pattern import PATTERN_FLARM_POSITION_COMMENT
from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS

from .base import BaseParser


class FlarmParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'flarm'
        self.position_pattern = re.compile(PATTERN_FLARM_POSITION_COMMENT)

    def parse_position(self, aprs_comment):
        ac_match = self.position_pattern.match(aprs_comment)

        def if_present(arg, func):
            result = ac_match.group(arg)
            return (func(result)) if result else None

        return {'address_type': if_present('details', lambda x: int(x, 16) & 0b00000011),
                'aircraft_type': if_present('details', lambda x: (int(x, 16) & 0b01111100) >> 2),
                'stealth': if_present('details', lambda x: (int(x, 16) & 0b10000000) >> 7 == 1),
                'address': if_present('address', lambda x: x),
                'climb_rate': if_present('climb_rate', lambda x: int(x) * FPM_TO_MS),
                'turn_rate': if_present('turn_rate', lambda x: float(x) * HPM_TO_DEGS),
                'signal_quality': if_present('signal_quality', float),
                'error_count': if_present('error_count', int),
                'frequency_offset': if_present('frequency_offset', float),
                'gps_quality': if_present('gps_quality', lambda _x: {
                    'horizontal': int(ac_match.group('gps_quality_horizontal')),
                    'vertical': int(ac_match.group('gps_quality_vertical'))}),
                'software_version': if_present('software_version', float),
                'hardware_version': if_present('hardware_version', lambda x: int(x, 16)),
                'real_address': if_present('real_address', lambda x: x),
                'signal_power': if_present('signal_power', float),
                }



import unittest

from ogn.parser.aprs_comment.ogn_parser import OgnParser


class TestStringMethods(unittest.TestCase):
    def test_fail_validation(self):
        self.assertEqual(OgnParser().parse_receiver_beacon("notAValidToken"), None)

    def test_v021(self):
        message = OgnParser().parse_receiver_beacon("v0.2.1 CPU:0.8 RAM:25.6/458.9MB NTP:0.1ms/+2.3ppm +51.9C RF:+26-1.4ppm/-0.25dB")

        self.assertEqual(message['version'], "0.2.1")
        self.assertEqual(message['cpu_load'], 0.8)
        self.assertEqual(message['free_ram'], 25.6)
        self.assertEqual(message['total_ram'], 458.9)
        self.assertEqual(message['ntp_error'], 0.1)
        self.assertEqual(message['rt_crystal_correction'], 2.3)
        self.assertEqual(message['cpu_temp'], 51.9)

        self.assertEqual(message['rec_crystal_correction'], 26)
        self.assertEqual(message['rec_crystal_correction_fine'], -1.4)
        self.assertEqual(message['rec_input_noise'], -0.25)

    def test_v022(self):
        message = OgnParser().parse_receiver_beacon("v0.2.2.x86 CPU:0.5 RAM:669.9/887.7MB NTP:1.0ms/+6.2ppm +52.0C RF:+0.06dB")
        self.assertEqual(message['platform'], 'x86')

    def test_v025(self):
        message = OgnParser().parse_receiver_beacon("v0.2.5.RPI-GPU CPU:0.8 RAM:287.3/458.7MB NTP:1.0ms/-6.4ppm 5.016V 0.534A +51.9C RF:+55+0.4ppm/-0.67dB/+10.8dB@10km[57282]")
        self.assertEqual(message['voltage'], 5.016)
        self.assertEqual(message['amperage'], 0.534)
        self.assertEqual(message['senders_signal'], 10.8)
        self.assertEqual(message['senders_messages'], 57282)

        message = OgnParser().parse_receiver_beacon("v0.2.5.ARM CPU:0.4 RAM:638.0/970.5MB NTP:0.2ms/-1.1ppm +65.5C 14/16Acfts[1h] RF:+45+0.0ppm/+3.88dB/+24.0dB@10km[143717]/+26.7dB@10km[68/135]")
        self.assertEqual(message['senders_visible'], 14)
        self.assertEqual(message['senders_total'], 16)
        self.assertEqual(message['senders_signal'], 24.0)
        self.assertEqual(message['senders_messages'], 143717)
        self.assertEqual(message['good_senders_signal'], 26.7)
        self.assertEqual(message['good_senders'], 68)
        self.assertEqual(message['good_and_bad_senders'], 135)


if __name__ == '__main__':
    unittest.main()



import unittest

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.aprs_comment.lt24_parser import LT24Parser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = LT24Parser().parse_position("id25387 +123fpm GPS")

        self.assertEqual(message['address'], "25387")
        self.assertAlmostEqual(message['climb_rate'], 123 * FPM_TO_MS, 2)
        self.assertEqual(message['source'], 'GPS')


if __name__ == '__main__':
    unittest.main()



import unittest

from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.aprs_comment.ogn_parser import OgnParser


class TestStringMethods(unittest.TestCase):
    def test_invalid_token(self):
        self.assertEqual(OgnParser().parse_aircraft_beacon("notAValidToken"), None)

    def test_basic(self):
        message = OgnParser().parse_aircraft_beacon("id0ADDA5BA -454fpm -1.1rot 8.8dB 0e +51.2kHz gps4x5 hear1084 hearB597 hearB598")

        self.assertEqual(message['address_type'], 2)
        self.assertEqual(message['aircraft_type'], 2)
        self.assertFalse(message['stealth'])
        self.assertEqual(message['address'], "DDA5BA")
        self.assertAlmostEqual(message['climb_rate'], -454 * FPM_TO_MS, 2)
        self.assertEqual(message['turn_rate'], -1.1 * HPM_TO_DEGS)
        self.assertEqual(message['signal_quality'], 8.8)
        self.assertEqual(message['error_count'], 0)
        self.assertEqual(message['frequency_offset'], 51.2)
        self.assertEqual(message['gps_quality'], {'horizontal': 4, 'vertical': 5})
        self.assertEqual(len(message['proximity']), 3)
        self.assertEqual(message['proximity'][0], '1084')
        self.assertEqual(message['proximity'][1], 'B597')
        self.assertEqual(message['proximity'][2], 'B598')

    def test_stealth(self):
        message = OgnParser().parse_aircraft_beacon("id0ADD1234 -454fpm -1.1rot 8.8dB 0e +51.2kHz gps4x5 hear1084 hearB597 hearB598")
        self.assertFalse(message['stealth'])

        message = OgnParser().parse_aircraft_beacon("id8ADD1234 -454fpm -1.1rot 8.8dB 0e +51.2kHz gps4x5 hear1084 hearB597 hearB598")
        self.assertTrue(message['stealth'])

    def test_v024(self):
        message = OgnParser().parse_aircraft_beacon("id21400EA9 -2454fpm +0.9rot 19.5dB 0e -6.6kHz gps1x1 s6.02 h0A rDF0C56")

        self.assertEqual(message['software_version'], 6.02)
        self.assertEqual(message['hardware_version'], 10)
        self.assertEqual(message['real_address'], "DF0C56")

    def test_v024_ogn_tracker(self):
        message = OgnParser().parse_aircraft_beacon("id07353800 +020fpm -14.0rot FL004.43 38.5dB 0e -2.9kHz")

        self.assertEqual(message['flightlevel'], 4.43)

    def test_v025(self):
        message = OgnParser().parse_aircraft_beacon("id06DDE28D +535fpm +3.8rot 11.5dB 0e -1.0kHz gps2x3 s6.01 h0C +7.4dBm")

        self.assertEqual(message['signal_power'], 7.4)

    def test_v026(self):
        # from 0.2.6 it is sufficent we have only the ID, climb and turn rate or just the ID
        message_triple = OgnParser().parse_aircraft_beacon("id093D0930 +000fpm +0.0rot")
        message_single = OgnParser().parse_aircraft_beacon("id093D0930")

        self.assertIsNotNone(message_triple)
        self.assertIsNotNone(message_single)


if __name__ == '__main__':
    unittest.main()



import re

from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.pattern import PATTERN_RECEIVER_BEACON, PATTERN_AIRCRAFT_BEACON

from .base import BaseParser


class OgnParser(BaseParser):
    def __init__(self):
        self.beacon_type = None
        self.aircraft_pattern = re.compile(PATTERN_AIRCRAFT_BEACON)
        self.receiver_pattern = re.compile(PATTERN_RECEIVER_BEACON)

    def parse(self, aprs_comment, aprs_type):
        if not aprs_comment:
            return {'beacon_type': 'aprs_receiver'}

        ac_data = self.parse_aircraft_beacon(aprs_comment)
        if ac_data:
            ac_data.update({'beacon_type': 'aprs_aircraft'})
            return ac_data

        rc_data = self.parse_receiver_beacon(aprs_comment)
        if rc_data:
            rc_data.update({'beacon_type': 'aprs_receiver'})
            return rc_data
        else:
            return {'user_comment': aprs_comment,
                    'beacon_type': 'aprs_receiver'}

    def parse_aircraft_beacon(self, aprs_comment):
        ac_match = self.aircraft_pattern.match(aprs_comment)
        if ac_match:
            return {'address_type': int(ac_match.group('details'), 16) & 0b00000011,
                    'aircraft_type': (int(ac_match.group('details'), 16) & 0b01111100) >> 2,
                    'stealth': (int(ac_match.group('details'), 16) & 0b10000000) >> 7 == 1,
                    'address': ac_match.group('address'),
                    'climb_rate': int(ac_match.group('climb_rate')) * FPM_TO_MS if ac_match.group('climb_rate') else None,
                    'turn_rate': float(ac_match.group('turn_rate')) * HPM_TO_DEGS if ac_match.group('turn_rate') else None,
                    'flightlevel': float(ac_match.group('flight_level')) if ac_match.group('flight_level') else None,
                    'signal_quality': float(ac_match.group('signal_quality')) if ac_match.group('signal_quality') else None,
                    'error_count': int(ac_match.group('errors')) if ac_match.group('errors') else None,
                    'frequency_offset': float(ac_match.group('frequency_offset')) if ac_match.group('frequency_offset') else None,
                    'gps_quality': {'horizontal': int(ac_match.group('gps_quality_horizontal')),
                                    'vertical': int(ac_match.group('gps_quality_vertical'))} if ac_match.group('gps_quality') else None,
                    'software_version': float(ac_match.group('flarm_software_version')) if ac_match.group('flarm_software_version') else None,
                    'hardware_version': int(ac_match.group('flarm_hardware_version'), 16) if ac_match.group('flarm_hardware_version') else None,
                    'real_address': ac_match.group('flarm_id') if ac_match.group('flarm_id') else None,
                    'signal_power': float(ac_match.group('signal_power')) if ac_match.group('signal_power') else None,
                    'proximity': [hear[4:] for hear in ac_match.group('proximity').split(" ")] if ac_match.group('proximity') else None}
        else:
            return None

    def parse_receiver_beacon(self, aprs_comment):
        rec_match = self.receiver_pattern.match(aprs_comment)
        if rec_match:
            return {'version': rec_match.group('version'),
                    'platform': rec_match.group('platform'),
                    'cpu_load': float(rec_match.group('cpu_load')),
                    'free_ram': float(rec_match.group('ram_free')),
                    'total_ram': float(rec_match.group('ram_total')),
                    'ntp_error': float(rec_match.group('ntp_offset')),
                    'rt_crystal_correction': float(rec_match.group('ntp_correction')),
                    'voltage': float(rec_match.group('voltage')) if rec_match.group('voltage') else None,
                    'amperage': float(rec_match.group('amperage')) if rec_match.group('amperage') else None,
                    'cpu_temp': float(rec_match.group('cpu_temperature')) if rec_match.group('cpu_temperature') else None,
                    'senders_visible': int(rec_match.group('visible_senders')) if rec_match.group('visible_senders') else None,
                    'senders_total': int(rec_match.group('senders')) if rec_match.group('senders') else None,
                    'rec_crystal_correction': int(rec_match.group('rf_correction_manual')) if rec_match.group('rf_correction_manual') else None,
                    'rec_crystal_correction_fine': float(rec_match.group('rf_correction_automatic')) if rec_match.group('rf_correction_automatic') else None,
                    'rec_input_noise': float(rec_match.group('signal_quality')) if rec_match.group('signal_quality') else None,
                    'senders_signal': float(rec_match.group('senders_signal_quality')) if rec_match.group('senders_signal_quality') else None,
                    'senders_messages': float(rec_match.group('senders_messages')) if rec_match.group('senders_messages') else None,
                    'good_senders_signal': float(rec_match.group('good_senders_signal_quality')) if rec_match.group('good_senders_signal_quality') else None,
                    'good_senders': float(rec_match.group('good_senders')) if rec_match.group('good_senders') else None,
                    'good_and_bad_senders': float(rec_match.group('good_and_bad_senders')) if rec_match.group('good_and_bad_senders') else None}
        else:
            return None



import re

from ogn.parser.pattern import PATTERN_TRACKER_POSITION_COMMENT, PATTERN_TRACKER_STATUS_COMMENT
from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS

from .base import BaseParser


class TrackerParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'tracker'
        self.position_pattern = re.compile(PATTERN_TRACKER_POSITION_COMMENT)
        self.status_pattern = re.compile(PATTERN_TRACKER_STATUS_COMMENT)

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        return {'address_type': int(match.group('details'), 16) & 0b00000011,
                'aircraft_type': (int(match.group('details'), 16) & 0b01111100) >> 2,
                'stealth': (int(match.group('details'), 16) & 0b10000000) >> 7 == 1,
                'address': match.group('address'),
                'climb_rate': int(match.group('climb_rate')) * FPM_TO_MS if match.group('climb_rate') else None,
                'turn_rate': float(match.group('turn_rate')) * HPM_TO_DEGS if match.group('turn_rate') else None,
                'flightlevel': float(match.group('flight_level')) if match.group('flight_level') else None,
                'signal_quality': float(match.group('signal_quality')) if match.group('signal_quality') else None,
                'error_count': int(match.group('error_count')) if match.group('error_count') else None,
                'frequency_offset': float(match.group('frequency_offset')) if match.group('frequency_offset') else None,
                'gps_quality': {'horizontal': int(match.group('gps_quality_horizontal')),
                                'vertical': int(match.group('gps_quality_vertical'))} if match.group('gps_quality') else None,
                'signal_power': float(match.group('signal_power')) if match.group('signal_power') else None}

    def parse_status(self, aprs_comment):
        match = self.status_pattern.match(aprs_comment)
        if match:
            return {'hardware_version': int(match.group('hardware_version')) if match.group('hardware_version') else None,
                    'software_version': int(match.group('software_version')) if match.group('software_version') else None,
                    'gps_satellites': int(match.group('gps_satellites')) if match.group('gps_satellites') else None,
                    'gps_quality': int(match.group('gps_quality')) if match.group('gps_quality') else None,
                    'gps_altitude': int(match.group('gps_altitude')) if match.group('gps_altitude') else None,
                    'pressure': float(match.group('pressure')) if match.group('pressure') else None,
                    'temperature': float(match.group('temperature')) if match.group('temperature') else None,
                    'humidity': int(match.group('humidity')) if match.group('humidity') else None,
                    'voltage': float(match.group('voltage')) if match.group('voltage') else None,
                    'transmitter_power': int(match.group('transmitter_power')) if match.group('transmitter_power') else None,
                    'noise_level': float(match.group('noise_level')) if match.group('noise_level') else None,
                    'relays': int(match.group('relays')) if match.group('relays') else None}
        else:
            return {'comment': aprs_comment}



import re

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.pattern import PATTERN_SKYLINES_POSITION_COMMENT

from .base import BaseParser


class SkylinesParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'skylines'
        self.position_pattern = re.compile(PATTERN_SKYLINES_POSITION_COMMENT)

    def parse_position(self, aprs_comment):
        ac_match = self.position_pattern.match(aprs_comment)
        return {'address': ac_match.group('id'),
                'climb_rate': int(ac_match.group('climb_rate')) * FPM_TO_MS if ac_match.group('climb_rate') else None}



import unittest
import unittest.mock as mock
import os

from datetime import datetime
from time import sleep

from ogn.parser.parse import parse
from ogn.parser.exceptions import AprsParseError


class TestStringMethods(unittest.TestCase):
    @unittest.skip("Not yet implemented")
    def test_wtf(self):
        parse("BELG>OGNSDR,TCPIP*,qAC,GLIDERN2:/133835h4509.60NI00919.20E&/A=000246")


if __name__ == '__main__':
    unittest.main()



import unittest

from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.aprs_comment.flarm_parser import FlarmParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = FlarmParser().parse_position("id21A8CBA8 -039fpm +0.1rot 3.5dB 2e -8.7kHz gps1x2 s6.09 h43 rDF0267")

        self.assertEqual(message['address_type'], 1)
        self.assertEqual(message['aircraft_type'], 8)
        self.assertFalse(message['stealth'])
        self.assertEqual(message['address'], "A8CBA8")
        self.assertAlmostEqual(message['climb_rate'], -39 * FPM_TO_MS, 2)
        self.assertEqual(message['turn_rate'], 0.1 * HPM_TO_DEGS)
        self.assertEqual(message['signal_quality'], 3.5)
        self.assertEqual(message['error_count'], 2)
        self.assertEqual(message['frequency_offset'], -8.7)
        self.assertEqual(message['gps_quality'], {'horizontal': 1, 'vertical': 2})
        self.assertEqual(message['software_version'], 6.09)
        self.assertEqual(message['hardware_version'], 67)
        self.assertEqual(message['real_address'], "DF0267")


if __name__ == '__main__':
    unittest.main()



from ogn.parser import parse as parse_module    # noqa: F40 --- only for test functions. Without this a mock of parse would mock the function instead of the module
from ogn.parser.parse import parse, parse_aprs, parse_comment   # noqa: F401
from ogn.parser.exceptions import ParseError, AprsParseError, OgnParseError     # noqa: F401



import re

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.pattern import PATTERN_FANET_POSITION_COMMENT, PATTERN_FANET_STATUS_COMMENT

from .base import BaseParser


class FanetParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'fanet'
        self.position_parser = PATTERN_FANET_POSITION_COMMENT
        self.status_parser = PATTERN_FANET_STATUS_COMMENT

    def parse_position(self, aprs_comment):
        ac_match = self.position_parser.match(aprs_comment)
        return {'address_type': int(ac_match.group('details'), 16) & 0b00000011 if ac_match.group('details') else None,
                'aircraft_type': (int(ac_match.group('details'), 16) & 0b01111100) >> 2 if ac_match.group('details') else None,
                'stealth': (int(ac_match.group('details'), 16) & 0b10000000) >> 7 == 1 if ac_match.group('details') else None,
                'address': ac_match.group('address') if ac_match.group('address') else None,
                'climb_rate': int(ac_match.group('climb_rate')) * FPM_TO_MS if ac_match.group('climb_rate') else None}

    def parse_status(self, aprs_comment):
        match = self.status_parser.match(aprs_comment)
        return {'user_name': match.group('user_name') if match.group('user_name') else None,
                'signal_quality': float(match.group('signal_quality')) if match.group('signal_quality') else None,
                'frequency_offset': float(match.group('frequency_offset')) if match.group('frequency_offset') else None,
                'error_count': int(match.group('error_count')) if match.group('error_count') else None}



import re

from ogn.parser.pattern import PATTERN_NAVITER_POSITION_COMMENT
from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS

from .base import BaseParser


class NaviterParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'naviter'
        self.position_pattern = re.compile(PATTERN_NAVITER_POSITION_COMMENT)

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        return {'stealth': (int(match.group('details'), 16) & 0b1000000000000000) >> 15 == 1,
                'do_not_track': (int(match.group('details'), 16) & 0b0100000000000000) >> 14 == 1,
                'aircraft_type': (int(match.group('details'), 16) & 0b0011110000000000) >> 10,
                'address_type': (int(match.group('details'), 16) & 0b0000001111110000) >> 4,
                'reserved': (int(match.group('details'), 16) & 0b0000000000001111),
                'address': match.group('address'),
                'climb_rate': int(match.group('climb_rate')) * FPM_TO_MS if match.group('climb_rate') else None,
                'turn_rate': float(match.group('turn_rate')) * HPM_TO_DEGS if match.group('turn_rate') else None}



import unittest
import unittest.mock as mock

from datetime import datetime

from ogn.parser.telnet_parser import parse
from ogn.parser.exceptions import ParseError


class TestStringMethods(unittest.TestCase):
    @unittest.skip("Not yet implemented")
    def test_telnet_fail_corrupt(self):
        with self.assertRaises(ParseError):
            parse('This is rubbish')

    @mock.patch('ogn.parser.telnet_parser.datetime')
    def test_telnet_parse_complete(self, datetime_mock):
        # set the utcnow-mock near to the time in the test string
        datetime_mock.utcnow.return_value = datetime(2015, 1, 1, 10, 0, 55)

        message = parse('0.181sec:868.394MHz:   1:2:DDA411 103010: [ +50.86800, +12.15279]deg  988m  +0.1m/s  25.7m/s 085.4deg  -3.5deg/sec 5 03x04m 01f_-12.61kHz  5.8/15.5dB/2 10e   30.9km 099.5deg  +1.1deg + ?     R     B8949')

        self.assertEqual(message['pps_offset'], 0.181)
        self.assertEqual(message['frequency'], 868.394)
        self.assertEqual(message['aircraft_type'], 1)
        self.assertEqual(message['address_type'], 2)
        self.assertEqual(message['address'], 'DDA411')
        self.assertEqual(message['timestamp'], datetime(2015, 1, 1, 10, 30, 10))
        self.assertEqual(message['latitude'], 50.868)
        self.assertEqual(message['longitude'], 12.15279)
        self.assertEqual(message['altitude'], 988)
        self.assertEqual(message['climb_rate'], 0.1)
        self.assertEqual(message['ground_speed'], 25.7)
        self.assertEqual(message['track'], 85.4)
        self.assertEqual(message['turn_rate'], -3.5)
        self.assertEqual(message['magic_number'], 5)  # the '5' is a magic number... 1 if ground_speed is 0.0m/s an 3 or 5 if airborne. Do you have an idea what it is?
        self.assertEqual(message['gps_status'], '03x04')
        self.assertEqual(message['channel'], 1)
        self.assertEqual(message['flarm_timeslot'], True)
        self.assertEqual(message['ogn_timeslot'], False)
        self.assertEqual(message['frequency_offset'], -12.61)
        self.assertEqual(message['decode_quality'], 5.8)
        self.assertEqual(message['signal_quality'], 15.5)
        self.assertEqual(message['demodulator_type'], 2)
        self.assertEqual(message['error_count'], 10)
        self.assertEqual(message['distance'], 30.9)
        self.assertEqual(message['bearing'], 99.5)
        self.assertEqual(message['phi'], 1.1)
        self.assertEqual(message['multichannel'], True)

    def test_telnet_parse_corrupt(self):
        message = parse('0.397sec:868.407MHz:  sA:1:784024 205656: [  +5.71003, +20.48951]deg 34012m +14.5m/s 109.7m/s 118.5deg +21.0deg/sec 0 27x40m 01_o +7.03kHz 17.2/27.0dB/2 12e 4719.5km 271.1deg  -8.5deg   ?     R     B34067')

        self.assertIsNone(message)


if __name__ == '__main__':
    unittest.main()



from datetime import datetime, timedelta

FEETS_TO_METER = 0.3048             # ratio feets to meter
FPM_TO_MS = FEETS_TO_METER / 60     # ratio fpm to m/s
KNOTS_TO_MS = 0.5144                # ratio knots to m/s
KPH_TO_MS = 0.27778                 # ratio kph to m/s
HPM_TO_DEGS = 180 / 60              # ratio between half turn per minute and degrees/s


def parseAngle(dddmmhht):
    return float(dddmmhht[:3]) + float(dddmmhht[3:]) / 60


def createTimestamp(time_string, reference_timestamp=None):
    if time_string[-1] == "z":
        dd = int(time_string[0:2])
        hh = int(time_string[2:4])
        mm = int(time_string[4:6])

        result = datetime(reference_timestamp.year,
                          reference_timestamp.month,
                          dd,
                          hh, mm, 0)

        if result > reference_timestamp + timedelta(days=14):
            # shift timestamp to previous month
            result = (result.replace(day=1) - timedelta(days=5)).replace(day=result.day)
        elif result < reference_timestamp - timedelta(days=14):
            # shift timestamp to next month
            result = (result.replace(day=28) + timedelta(days=5)).replace(day=result.day)
    else:
        hh = int(time_string[0:2])
        mm = int(time_string[2:4])
        ss = int(time_string[4:6])

        result = datetime(reference_timestamp.year,
                          reference_timestamp.month,
                          reference_timestamp.day,
                          hh, mm, ss)

        if result > reference_timestamp + timedelta(hours=12):
            # shift timestamp to previous day
            result -= timedelta(days=1)
        elif result < reference_timestamp - timedelta(hours=12):
            # shift timestamp to next day
            result += timedelta(days=1)

    return result



import unittest

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.aprs_comment.fanet_parser import FanetParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = FanetParser().parse_position("id1E1103CE -02fpm")

        self.assertEqual(message['address_type'], 2)
        self.assertEqual(message['aircraft_type'], 7)
        self.assertFalse(message['stealth'])
        self.assertEqual(message['address'], "1103CE")
        self.assertAlmostEqual(message['climb_rate'], -2 * FPM_TO_MS, 0.1)

    def test_pseudo_status_comment(self):
        message = FanetParser().parse_position("")

        self.assertIsNone(message['address_type'])
        self.assertIsNone(message['aircraft_type'])
        self.assertIsNone(message['stealth'])
        self.assertIsNone(message['address'])
        self.assertIsNone(message['climb_rate'])

    def test_status_comment(self):
        message = FanetParser().parse_status("""Name="Roman" 16.8dB -16.7kHz 1e""")

        self.assertEqual(message['user_name'], 'Roman')
        self.assertEqual(message['signal_quality'], 16.8)
        self.assertEqual(message['frequency_offset'], -16.7)
        self.assertEqual(message['error_count'], 1)

    def test_status_comment2(self):
        message = FanetParser().parse_status("""Name="Thomas Sauter" 14.3dB -12.0kHz""")

        self.assertEqual(message['user_name'], 'Thomas Sauter')
        self.assertEqual(message['signal_quality'], 14.3)
        self.assertEqual(message['frequency_offset'], -12.0)

    def test_status_comment3(self):
        message = FanetParser().parse_status("""Name="Joerg Kaiser" 14.3dB""")

        self.assertEqual(message['user_name'], 'Joerg Kaiser')
        self.assertEqual(message['signal_quality'], 14.3)


if __name__ == '__main__':
    unittest.main()



import re

from ogn.parser.pattern import PATTERN_INREACH_POSITION_COMMENT

from .base import BaseParser


class InreachParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'inreach'
        self.position_pattern = re.compile(PATTERN_INREACH_POSITION_COMMENT)

    def parse_position(self, aprs_comment):
        ac_match = self.position_pattern.match(aprs_comment)
        return {'address': ac_match.group('id'),
                'model': ac_match.group('model') if ac_match.group('model') else None,
                'status': ac_match.group('status') == 'True' if ac_match.group('status') else None,
                'pilot_name': ac_match.group('pilot_name') if ac_match.group('pilot_name') else None}



import unittest

from ogn.ddb import get_ddb_devices


class TestStringMethods(unittest.TestCase):
    def test_get_ddb_devices(self):
        devices = list(get_ddb_devices())
        self.assertGreater(len(devices), 4000)
        self.assertCountEqual(devices[0].keys(), ['device_type', 'device_id', 'aircraft_model', 'registration', 'cn', 'tracked', 'identified'])



#!/usr/bin/env python3

from os import path
from setuptools import setup, find_packages

from ogn.client.settings import PACKAGE_VERSION


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ogn-client',
    version=PACKAGE_VERSION,
    description='A python module for the Open Glider Network',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/glidernet/python-ogn-client',
    author='Konstantin Gründger aka Meisterschueler, Fabian P. Schmidt aka kerel',
    author_email='kerel-fs@gmx.de',
    license='AGPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: GIS',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    keywords='gliding ogn',
    packages=['ogn.{}'.format(package) for package in find_packages(where='ogn')],
    python_requires='>=3',
    install_requires=[],
    extras_require={
        'dev': [
            'nose==1.3.7',
            'coveralls==2.0.0',
            'flake8==3.8.1'
        ]
    },
    zip_safe=False
)



import re

from ogn.parser.pattern import PATTERN_RECEIVER_POSITION_COMMENT, PATTERN_RECEIVER_STATUS_COMMENT

from .base import BaseParser


class ReceiverParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'receiver'
        self.position_pattern = re.compile(PATTERN_RECEIVER_POSITION_COMMENT)
        self.status_pattern = re.compile(PATTERN_RECEIVER_STATUS_COMMENT)

    def parse_position(self, aprs_comment):
        if aprs_comment is None:
            return {}
        else:
            match = self.position_pattern.match(aprs_comment)
            return {'user_comment': match.group('user_comment') if match.group('user_comment') else None}

    def parse_status(self, aprs_comment):
        match = self.status_pattern.match(aprs_comment)
        return {'version': match.group('version'),
                'platform': match.group('platform'),
                'cpu_load': float(match.group('cpu_load')),
                'free_ram': float(match.group('ram_free')),
                'total_ram': float(match.group('ram_total')),
                'ntp_error': float(match.group('ntp_offset')),
                'rt_crystal_correction': float(match.group('ntp_correction')),
                'voltage': float(match.group('voltage')) if match.group('voltage') else None,
                'amperage': float(match.group('amperage')) if match.group('amperage') else None,
                'cpu_temp': float(match.group('cpu_temperature')) if match.group('cpu_temperature') else None,
                'senders_visible': int(match.group('visible_senders')) if match.group('visible_senders') else None,
                'senders_total': int(match.group('senders')) if match.group('senders') else None,
                'rec_crystal_correction': int(match.group('rf_correction_manual')) if match.group('rf_correction_manual') else None,
                'rec_crystal_correction_fine': float(match.group('rf_correction_automatic')) if match.group('rf_correction_automatic') else None,
                'rec_input_noise': float(match.group('signal_quality')) if match.group('signal_quality') else None,
                'senders_signal': float(match.group('senders_signal_quality')) if match.group('senders_signal_quality') else None,
                'senders_messages': float(match.group('senders_messages')) if match.group('senders_messages') else None,
                'good_senders_signal': float(match.group('good_senders_signal_quality')) if match.group('good_senders_signal_quality') else None,
                'good_senders': float(match.group('good_senders')) if match.group('good_senders') else None,
                'good_and_bad_senders': float(match.group('good_and_bad_senders')) if match.group('good_and_bad_senders') else None}



import re

from ogn.parser.pattern import PATTERN_SPIDER_POSITION_COMMENT

from .base import BaseParser


class SpiderParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'spider'
        self.position_pattern = re.compile(PATTERN_SPIDER_POSITION_COMMENT)

    def parse_position(self, aprs_comment):
        ac_match = self.position_pattern.match(aprs_comment)
        return {'address': ac_match.group('id'),
                'signal_power': int(ac_match.group('signal_power')) if ac_match.group('signal_power') else None,
                'spider_id': ac_match.group('spider_id') if ac_match.group('spider_id') else None,
                'gps_quality': ac_match.group('gps_quality') if ac_match.group('gps_quality') else None}



import unittest
import unittest.mock as mock
import os

from datetime import datetime
from time import sleep

from ogn.parser.parse import parse
from ogn.parser.exceptions import AprsParseError


class TestStringMethods(unittest.TestCase):
    def parse_valid_beacon_data_file(self, filename, beacon_type):
        with open(os.path.dirname(__file__) + '/valid_beacon_data/' + filename) as f:
            for line in f:
                try:
                    message = parse(line, datetime(2015, 4, 10, 17, 0))
                    self.assertFalse(message is None)
                    if message['aprs_type'] == 'position' or message['aprs_type'] == 'status':
                        self.assertEqual(message['beacon_type'], beacon_type)
                except NotImplementedError as e:
                    print(e)

    def test_aprs_aircraft_beacons(self):
        self.parse_valid_beacon_data_file(filename='aprs_aircraft.txt', beacon_type='aprs_aircraft')

    def test_aprs_receiver_beacons(self):
        self.parse_valid_beacon_data_file(filename='aprs_receiver.txt', beacon_type='aprs_receiver')

    def test_aprs_fanet_beacons(self):
        self.parse_valid_beacon_data_file(filename='fanet.txt', beacon_type='fanet')

    def test_flarm_beacons(self):
        self.parse_valid_beacon_data_file(filename='flarm.txt', beacon_type='flarm')

    def test_receiver_beacons(self):
        self.parse_valid_beacon_data_file(filename='receiver.txt', beacon_type='receiver')

    def test_tracker_beacons(self):
        self.parse_valid_beacon_data_file(filename='tracker.txt', beacon_type='tracker')

    def test_capturs_beacons(self):
        self.parse_valid_beacon_data_file(filename='capturs.txt', beacon_type='capturs')

    def test_flymaster_beacons(self):
        self.parse_valid_beacon_data_file(filename='flymaster.txt', beacon_type='flymaster')

    def test_inreach_beacons(self):
        self.parse_valid_beacon_data_file(filename='inreach.txt', beacon_type='inreach')

    def test_lt24_beacons(self):
        self.parse_valid_beacon_data_file(filename='lt24.txt', beacon_type='lt24')

    def test_naviter_beacons(self):
        self.parse_valid_beacon_data_file(filename='naviter.txt', beacon_type='naviter')

    def test_pilot_aware_beacons(self):
        self.parse_valid_beacon_data_file(filename='pilot_aware.txt', beacon_type='pilot_aware')

    def test_skylines_beacons(self):
        self.parse_valid_beacon_data_file(filename='skylines.txt', beacon_type='skylines')

    def test_spider_beacons(self):
        self.parse_valid_beacon_data_file(filename='spider.txt', beacon_type='spider')

    def test_spot_beacons(self):
        self.parse_valid_beacon_data_file(filename='spot.txt', beacon_type='spot')

    def test_generic_beacons(self):
        message = parse("EPZR>WTFDSTCALL,TCPIP*,qAC,GLIDERN1:>093456h this is a comment")
        self.assertEqual(message['beacon_type'], 'unknown')
        self.assertEqual(message['comment'], "this is a comment")

    def test_fail_parse_aprs_none(self):
        with self.assertRaises(TypeError):
            parse(None)

    def test_fail_empty(self):
        with self.assertRaises(AprsParseError):
            parse("")

    def test_fail_bad_string(self):
        with self.assertRaises(AprsParseError):
            parse("Lachens>APRS,TCPIwontbeavalidstring")

    def test_v026_chile(self):
        # receiver beacons from chile have a APRS position message with a pure user comment
        message = parse("VITACURA1>APRS,TCPIP*,qAC,GLIDERN4:/201146h3322.79SI07034.80W&/A=002329 Vitacura Municipal Aerodrome, Club de Planeadores Vitacura")

        self.assertEqual(message['user_comment'], "Vitacura Municipal Aerodrome, Club de Planeadores Vitacura")

        message_with_id = parse("ALFALFAL>APRS,TCPIP*,qAC,GLIDERN4:/221830h3330.40SI07007.88W&/A=008659 Alfalfal Hidroelectric Plant, Club de Planeadores Vitacurs")

        self.assertEqual(message_with_id['user_comment'], "Alfalfal Hidroelectric Plant, Club de Planeadores Vitacurs")

    @mock.patch('ogn.parser.parse_module.createTimestamp')
    def test_default_reference_date(self, createTimestamp_mock):
        valid_aprs_string = "Lachens>APRS,TCPIP*,qAC,GLIDERN2:/165334h4344.70NI00639.19E&/A=005435 v0.2.1 CPU:0.3 RAM:1764.4/2121.4MB NTP:2.8ms/+4.9ppm +47.0C RF:+0.70dB"

        parse(valid_aprs_string)
        call_args_before = createTimestamp_mock.call_args

        sleep(1)

        parse(valid_aprs_string)
        call_args_seconds_later = createTimestamp_mock.call_args

        self.assertNotEqual(call_args_before, call_args_seconds_later)

    def test_copy_constructor(self):
        valid_aprs_string = "FLRDDA5BA>APRS,qAS,LFMX:/160829h4415.41N/00600.03E'342/049/A=005524 id0ADDA5BA -454fpm -1.1rot 8.8dB 0e +51.2kHz gps4x5"
        message = parse(valid_aprs_string)

        self.assertEqual(message['name'], 'FLRDDA5BA')
        self.assertEqual(message['address'], 'DDA5BA')


if __name__ == '__main__':
    unittest.main()



import unittest

from datetime import datetime

from ogn.parser.utils import KNOTS_TO_MS, KPH_TO_MS, FEETS_TO_METER
from ogn.parser.parse import parse_aprs
from ogn.parser.exceptions import AprsParseError


class TestStringMethods(unittest.TestCase):
    def test_fail_validation(self):
        with self.assertRaises(AprsParseError):
            parse_aprs("notAValidString")

    def test_basic(self):
        message = parse_aprs("FLRDDA5BA>APRS,qAS,LFMX:/160829h4415.41N/00600.03E'342/049/A=005524 this is a comment")
        self.assertEqual(message['name'], "FLRDDA5BA")
        self.assertEqual(message['dstcall'], "APRS")
        self.assertEqual(message['receiver_name'], "LFMX")
        self.assertEqual(message['timestamp'].strftime('%H:%M:%S'), "16:08:29")
        self.assertAlmostEqual(message['latitude'], 44.25683, 5)
        self.assertEqual(message['symboltable'], '/')
        self.assertAlmostEqual(message['longitude'], 6.0005, 5)
        self.assertEqual(message['symbolcode'], '\'')
        self.assertEqual(message['track'], 342)
        self.assertEqual(message['ground_speed'], 49 * KNOTS_TO_MS / KPH_TO_MS)
        self.assertAlmostEqual(message['altitude'], 5524 * FEETS_TO_METER, 5)
        self.assertEqual(message['comment'], "this is a comment")

        self.assertEqual(message['aprs_type'], 'position')

    def test_v024(self):
        # higher precision datum format introduced
        raw_message = "FLRDDA5BA>APRS,qAS,LFMX:/160829h4415.41N/00600.03E'342/049/A=005524 !W26! id21400EA9 -2454fpm +0.9rot 19.5dB 0e -6.6kHz gps1x1 s6.02 h44 rDF0C56"
        message = parse_aprs(raw_message)

        self.assertAlmostEqual(message['latitude'] - 44.2568 - 1 / 30000, 2 / 1000 / 60, 10)
        self.assertAlmostEqual(message['longitude'] - 6.0005, 6 / 1000 / 60, 10)

    def test_v025(self):
        # introduced the "aprs status" format where many informations (lat, lon, alt, speed, ...) are just optional
        raw_message = "EPZR>APRS,TCPIP*,qAC,GLIDERN1:>093456h this is a comment"
        message = parse_aprs(raw_message)

        self.assertEqual(message['name'], "EPZR")
        self.assertEqual(message['receiver_name'], "GLIDERN1")
        self.assertEqual(message['timestamp'].strftime('%H:%M:%S'), "09:34:56")
        self.assertEqual(message['comment'], "this is a comment")

        self.assertEqual(message['aprs_type'], 'status')

    def test_v026(self):
        # from 0.2.6 the ogn comment of a receiver beacon is just optional
        raw_message = "Ulrichamn>APRS,TCPIP*,qAC,GLIDERN1:/085616h5747.30NI01324.77E&/A=001322"
        message = parse_aprs(raw_message)

        self.assertEqual(message['comment'], '')

    def test_v026_relay(self):
        # beacons can be relayed
        raw_message = "FLRFFFFFF>OGNAVI,NAV07220E*,qAS,NAVITER:/092002h1000.00S/01000.00W'000/000/A=003281 !W00! id2820FFFFFF +300fpm +1.7rot"
        message = parse_aprs(raw_message)

        self.assertEqual(message['relay'], "NAV07220E")

    def test_v027_ddhhmm(self):
        # beacons can have hhmmss or ddhhmm timestamp
        raw_message = "ICA4B0678>APRS,qAS,LSZF:/301046z4729.50N/00812.89E'227/091/A=002854 !W01! id054B0678 +040fpm +0.0rot 19.0dB 0e +1.5kHz gps1x1"
        message = parse_aprs(raw_message)

        self.assertEqual(message['timestamp'].strftime('%d %H:%M'), "30 10:46")

    def test_negative_altitude(self):
        # some devices can report negative altitudes
        raw_message = "OGNF71F40>APRS,qAS,NAVITER:/080852h4414.37N/01532.06E'253/052/A=-00013 !W73! id1EF71F40 -060fpm +0.0rot"
        message = parse_aprs(raw_message)

        self.assertAlmostEqual(message['altitude'], -13 * FEETS_TO_METER, 5)

    def test_no_altitude(self):
        # altitude is not a 'must have'
        raw_message = "FLRDDEEF1>OGCAPT,qAS,CAPTURS:/065511h4837.63N/00233.79E'000/000/"
        message = parse_aprs(raw_message)

        self.assertEqual(message['altitude'], None)

    def test_invalid_coordinates(self):
        # sometimes the coordinates leave their valid range: -90<=latitude<=90 or -180<=longitude<=180
        with self.assertRaises(AprsParseError):
            parse_aprs("RND000000>APRS,qAS,TROCALAN1:/210042h6505.31S/18136.75W^054/325/A=002591 !W31! idA4000000 +099fpm +1.8rot FL029.04 12.0dB 5e -6.3kHz gps11x17")

        with self.assertRaises(AprsParseError):
            parse_aprs("RND000000>APRS,qAS,TROCALAN1:/210042h9505.31S/17136.75W^054/325/A=002591 !W31! idA4000000 +099fpm +1.8rot FL029.04 12.0dB 5e -6.3kHz gps11x17")

    def test_invalid_timestamp(self):
        with self.assertRaises(AprsParseError):
            parse_aprs("OGND4362A>APRS,qAS,Eternoz:/194490h4700.25N/00601.47E'003/063/A=000000 !W22! id07D4362A 0fpm +0.0rot FL000.00 2.0dB 3e -2.8kHz gps3x4 +12.2dBm")

        with self.assertRaises(AprsParseError):
            parse_aprs("Ulrichamn>APRS,TCPIP*,qAC,GLIDERN1:/194490h5747.30NI01324.77E&/A=001322")

    def test_invalid_altitude(self):
        with self.assertRaises(AprsParseError):
            parse_aprs("Ulrichamn>APRS,TCPIP*,qAC,GLIDERN1:/085616h5747.30NI01324.77E&/A=12-345")

    def test_bad_comment(self):
        raw_message = "# bad configured ogn receiver"
        message = parse_aprs(raw_message)

        self.assertEqual(message['comment'], raw_message)
        self.assertEqual(message['aprs_type'], 'comment')

    def test_server_comment(self):
        raw_message = "# aprsc 2.1.4-g408ed49 17 Mar 2018 09:30:36 GMT GLIDERN1 37.187.40.234:10152"
        message = parse_aprs(raw_message)

        self.assertEqual(message['version'], '2.1.4-g408ed49')
        self.assertEqual(message['timestamp'], datetime(2018, 3, 17, 9, 30, 36))
        self.assertEqual(message['server'], 'GLIDERN1')
        self.assertEqual(message['ip_address'], '37.187.40.234')
        self.assertEqual(message['port'], '10152')
        self.assertEqual(message['aprs_type'], 'server')


if __name__ == '__main__':
    unittest.main()
